﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Text.RegularExpressions;
using System.IO.Compression;

namespace HttpWatch
{
    public partial class MainForm : Form
    {
        private HttpWatch watch = null;

        private List<string> cacheList = new List<string>();

        public MainForm()
        {
            InitializeComponent();
            Control.CheckForIllegalCrossThreadCalls = false;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            this.lstUrl.Columns.Add("地址", 370, HorizontalAlignment.Left);

            Thread t = new Thread(new ThreadStart(Execute));
            t.IsBackground = true;
            t.Start();
        }

        private void Execute()
        {
            watch = new HttpWatch("192.168.0.29");
            watch.Start();

            while (true)
            {
                try
                {
                    var dic = watch.GetDic();
                    foreach (var item in dic)
                    {
                        httpsession osession = dic[item.Key];
                        if (osession.responseraw == null)
                        {
                            continue;
                        }

                        if (!cacheList.Contains(item.Key))
                        {
                            cacheList.Add(item.Key);
                        }
                        else
                        {
                            continue;
                        }

                        byte[] dataall = osession.responseraw.ToArray();
                        byte[] responcebody = dataall;
                        byte[] requestbody = osession.sendraw.ToArray();
                        int tag = 0;
                        for (int i = 0; i < dataall.Length - 3; i++)
                        {
                            if (dataall[i] == 0x0D && dataall[i + 1] == 0x0A && dataall[i + 2] == 0x0D && dataall[i + 3] == 0x0A)
                            {
                                tag = i;
                                break;
                            }
                        }

                        byte[] headbyte = new byte[tag + 4];
                        Array.Copy(dataall, 0, headbyte, 0, tag + 4);
                        byte[] recbyte = new byte[dataall.Length - headbyte.Length];
                        Array.Copy(dataall, tag + 4, recbyte, 0, recbyte.Length);
                        string headstring = Encoding.ASCII.GetString(headbyte).Trim(new char[] { '\0' });

                        if (headstring.IndexOf("chunk") >= 0)
                        {
                            recbyte = Chunk.doUnchunk(recbyte);
                        }

                        System.IO.Stream responseStream = new System.IO.MemoryStream(recbyte);
                        string compress = getencompress(headstring);
                        if (compress == "gzip")
                        {
                            responseStream = new GZipStream(responseStream, CompressionMode.Decompress);
                        }
                        else if (compress == "deflate")// if (headstring.ToLower().Contains("deflate"))
                        {
                            responseStream = new DeflateStream(responseStream, CompressionMode.Decompress);
                        }

                        if (isimage(headstring)) //图形
                        {
                            byte[] imgview_body = recbyte;
                        }
                        else
                        {
                            string encstring = getencfromhead(headstring);
                            if (encstring == "")
                            {
                                string prs = System.Text.Encoding.ASCII.GetString(recbyte);
                                encstring = getencfromhead(prs);
                            }
                            if (encstring == "")
                                encstring = "utf-8";
                            Encoding enc = Encoding.GetEncoding(encstring);
                            System.IO.StreamReader sr = new System.IO.StreamReader(responseStream, enc);

                            string html = sr.ReadToEnd();
                            string result = headstring + html;
                            this.lstUrl.BeginUpdate();
                            this.lstUrl.Items.Add(new ListViewItem(item.Value.url) { Tag = result });
                            this.lstUrl.EndUpdate();
                        }
                    }
                }
                catch
                {
                }
                finally
                {
                    Thread.Sleep(1000);
                }
            }
        }

        private bool isimage(string head)
        {
            if (head.ToLower().IndexOf("image") > 0)
                return true;
            return false;
        }

        private string getencompress(string head)
        {
            if (head.ToLower().IndexOf("gzip") > 0)
                return "gzip";
            if (head.ToLower().IndexOf("deflate") > 0)
                return "deflate";
            return "";
        }

        Regex regexencode = new Regex("charset=([\\w|-]+)", RegexOptions.IgnoreCase);
        private string getencfromhead(string head)
        {
            Match mc = regexencode.Match(head);
            if (mc.Success)
            {
                return mc.Groups[1].Value;
            }

            return string.Empty;
        }

        private void lstUrl_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (this.lstUrl.SelectedItems != null && this.lstUrl.SelectedItems.Count > 0)
            {
                string text = this.lstUrl.SelectedItems[0].Tag as string;
                this.txtContent.Text = text;
            }
        }
    }
}
